

#include "SaveFile.H"
#include "gba.H"


using namespace avelino;
using namespace std;


const char SaveFile::SIGNATURE[SaveFile::SIGNATURE_SIZE + 1] = "RollingBlocks";


void SaveFile::init(Position position) {
	this->position = position;
	this->levels[0] = LevelStatus::UNLOCKED;     // first 3 levels unlocked, rest of levels locked
	this->levels[1] = LevelStatus::UNLOCKED;
	this->levels[2] = LevelStatus::UNLOCKED;
	uint16_t n = LevelManager::NUM_LEVELS - 3;
	LevelStatus *p = this->levels + 3;
	while (n > 0) {
		*p = LevelStatus::LOCKED;
		n--;
		p++;
	}
}


uint8_t *SaveFileManager::getData(SaveFile::Position position) {
	if (position == SaveFile::Position::FILE_1)
		return GAMEPAK_SRAM;
	else if (position == SaveFile::Position::FILE_2)
		return GAMEPAK_SRAM + 1024;
	else
		return GAMEPAK_SRAM + 2048;
}


void SaveFileManager::startSRAMAccess() {
	this->waitcntBackup = WAITCNT;
	WAITCNT = 3;
}


void SaveFileManager::endSRAMAccess() {
	WAITCNT = this->waitcntBackup;
}


bool SaveFileManager::isInUse(SaveFile::Position position) {
	uint8_t *p = this->getData(position);
	this->startSRAMAccess();
	bool ret = memequal(p, (const uint8_t *) SaveFile::SIGNATURE, SaveFile::SIGNATURE_SIZE);
	this->endSRAMAccess();
	return ret;
}


void SaveFileManager::init(SaveFile::Position position, SaveFile &file) {
	uint8_t *p = this->getData(position);
	this->startSRAMAccess();
	memcpy8(p, (const uint8_t *) SaveFile::SIGNATURE, SaveFile::SIGNATURE_SIZE);
	file.init(position);
	memcpy8(p + SaveFile::SIGNATURE_SIZE, (const uint8_t *) &file, sizeof(SaveFile));
	this->endSRAMAccess();
}


void SaveFileManager::load(SaveFile::Position position, SaveFile &file) {
	uint8_t *p = this->getData(position);
	this->startSRAMAccess();
	memcpy8((uint8_t *) &file, p + SaveFile::SIGNATURE_SIZE, sizeof(SaveFile));
	this->endSRAMAccess();
}


void SaveFileManager::save(SaveFile::Position position, SaveFile &file) {
	uint8_t *p = this->getData(position);
	this->startSRAMAccess();
	memcpy8(p, (const uint8_t *) SaveFile::SIGNATURE, SaveFile::SIGNATURE_SIZE);
	memcpy8(p + SaveFile::SIGNATURE_SIZE, (const uint8_t *) &file, sizeof(SaveFile));
	this->endSRAMAccess();
}


void SaveFileManager::remove(SaveFile::Position position) {
	uint8_t *p = this->getData(position);
	this->startSRAMAccess();
	memset8(p, 0, SaveFile::SIGNATURE_SIZE + sizeof(SaveFile));
	this->endSRAMAccess();
}

