

#ifndef  __SCREENBOARD_H__
#define  __SCREENBOARD_H__


#include "Screen.H"
#include "Level.H"
#include "SecondsTimer.H"


extern "C++" {
	namespace avelino {
		using namespace std;

		class ScreenBoard : public Screen, public SecondsTimerListener {
			protected:
				InterScreenData *interScreenData;
				class Direction {
					public:
						int16_t x;
						int16_t y;
				};
				static const uint16_t NUM_DIRECTIONS = 9;
				static const Direction DIRECTIONS[NUM_DIRECTIONS];
				static const uint16_t DIRECTION_TOP          = 0;
				static const uint16_t DIRECTION_BOTTOM       = 1;
				static const uint16_t DIRECTION_LEFT         = 2;
				static const uint16_t DIRECTION_RIGHT        = 3;
				static const uint16_t DIRECTION_TOP_LEFT     = 4;
				static const uint16_t DIRECTION_TOP_RIGHT    = 5;
				static const uint16_t DIRECTION_BOTTOM_LEFT  = 6;
				static const uint16_t DIRECTION_BOTTOM_RIGHT = 7;
				static const uint16_t DIRECTION_SELF         = 8;
				enum class ConsiderTheBlocks { NO, YES };
				bool isBoardNeighbourPassable(const Level &level, int16_t x, int16_t y, const Direction &d, ConsiderTheBlocks considerTheBlocks = ConsiderTheBlocks::NO);
				enum class TileIndex { TILE_0, TILE_1, TILE_2, TILE_3 };
				void drawBoard(const Level &level, uint16_t *destBG, uint16_t *destBGForEndings);
				void drawBoardBorder(const Level &level, uint16_t *destBG);
				void drawBoardBorder2(const Level &level, uint16_t *destBG);
				uint16_t getTile(TileIndex tileIndex, bool tlPassable, bool tPassable, bool trPassable, bool rPassable, bool brPassable, bool bPassable, bool blPassable, bool lPassable);
				static const uint16_t TILE_0_MAP[8];
				static const uint16_t TILE_1_MAP[8];
				static const uint16_t TILE_2_MAP[8];
				static const uint16_t TILE_3_MAP[8];
				static const uint16_t BOARD_TILES_OFFSET = 96;
				void drawFinish(int16_t x, int16_t y, uint16_t color, uint16_t *destBG);
				SecondsTimer secondsTimer;
				uint16_t remainMinutes;
				uint16_t remainSeconds;
				static const uint16_t NUM_BLOCKS = 4;
				Direction blockLocations[NUM_BLOCKS];
				bool blockEnabled[NUM_BLOCKS];
				uint16_t markerIndex;
				void updateMarkerFromIndex();
				void hideSprites();
				void drawText(const char *s, int16_t x, int16_t y, uint16_t *destBG);
				void hideBlock(uint16_t index);
				void locateBlock(uint16_t index, int16_t x, int16_t y);
				void locateBlocks(Level &level, uint16_t &firstAvailableBlockIndex);
				void locateMarker(int16_t x, int16_t y);
				void startMarker();
				enum class Movement { UP, DOWN, LEFT, RIGHT };
				void moveMarker(Movement m);
				int16_t xBoardSrc;
				int16_t yBoardSrc;
				int16_t xBoardDest;
				int16_t yBoardDest;
				int32_t xFP;
				int32_t yFP;
				int32_t xDestFP;
				int32_t yDestFP;
				int32_t xSpeedFP;
				int32_t ySpeedFP;
				static const int32_t BLOCK_SPEED_FP = (((int32_t) 3) << 16);
				void startMoveBlockOnMarker(Movement m);
				void updateMoveBlockOnMarker(bool &finished);
				bool levelFinished();
				void commitMovement();
				enum class Status { PRE_WAIT, MARK, STEER, MOVE, END_LOSE, END_WIN, ENDING, FINISHED };
				Status status;
				uint16_t markTiltCounter;
				static const uint16_t MARK_TILT_COUNTER_MAX = 40;
				static const uint16_t MARK_TILT_COUNTER_HIDE = 20;
				uint16_t waitKeyCounter;
				uint16_t waitCounter;
				static const uint16_t WAIT_KEY_COUNTER_MAX = 30;
				void configureAndHideArrows();
				void hideArrows();
				static const uint16_t ARROW_UP_MASK = 1;
				static const uint16_t ARROW_DOWN_MASK = 2;
				static const uint16_t ARROW_LEFT_MASK = 4;
				static const uint16_t ARROW_RIGHT_MASK = 8;
				uint16_t arrowsMask;
				void showArrowsAndUpdateArrowsMask();
				void updateArrowsMask();
				void showArrowsFromArrowsMask();
				void showRemainTime();
				enum class FinishMessage { YOU_LOSE, YOU_WIN };
				void initFinishMessage(FinishMessage fm);
				int32_t endingMessageYFP;
				static const int32_t ENDING_MESSAGE_SPEED_FP = (((int32_t) 1) << 16);
				static const uint16_t WAIT_COUNTER_AFTER_FINISH = (3 * 60);
				void saveLevelAsFinished();
			public:
				virtual void oneSecondElapsed(SecondsTimer &timer);
				Screen *pauseScreen;
				Screen *afterFinishScreen;
				virtual void onLoad(InterScreenData *dataFromPreviousScreen);
				virtual Screen *onVBlank(Keypad &keypad);
				virtual InterScreenData *onUnload();
		};
	}
}


#endif  // __SCREENBOARD_H__
