

#include "ScreenBoardSelectionMenu.H"
#include "gba.H"
#include "fonts.H"
#include "palette.H"
#include "tiles.H"
#include "GameStatus.H"
#include "Level.H"
#include "paths.H"


using namespace avelino;
using namespace std;


const ScreenBoardSelectionMenu::Location ScreenBoardSelectionMenu::LOCATIONS[LevelManager::NUM_LEVELS + 1] = {
	{5, 3, -1, 4}, {11, 3, -1, 5}, {17, 3, -1, 6}, {23, 3, -1, 7},            // all the 32 levels (boards)
	{5, 5, 0, 8}, {11, 5, 1, 9}, {17, 5, 2, 10}, {23, 5, 3, 11},
	{5, 7, 4, 12}, {11, 7, 5, 13}, {17, 7, 6, 14}, {23, 7, 7, 15},
	{5, 9, 8, 16}, {11, 9, 9, 17}, {17, 9, 10, 18}, {23, 9, 11, 19},
	{5, 11, 12, 20}, {11, 11, 13, 21}, {17, 11, 14, 22}, {23, 11, 15, 23},
	{5, 13, 16, 24}, {11, 13, 17, 25}, {17, 13, 18, 26}, {23, 13, 19, 27},
	{5, 15, 20, 28}, {11, 15, 21, 29}, {17, 15, 22, 30}, {23, 15, 23, 31},
	{5, 17, 24, 32}, {11, 17, 25, 32}, {17, 17, 26, 32}, {23, 17, 27, 32},
	{10, 19, 28, -1}                                                           // the "delete file" option
};


void ScreenBoardSelectionMenu::updatePath(PathManager &source, const PathPoint &point) {
	BG0HOFS = point.x & 0x01FF;
	BG0VOFS = point.y & 0x01FF;
}


void ScreenBoardSelectionMenu::updateTilt(TiltManager &tiltManager, int32_t v) {
	OAM[1] = (v >> 16) & 0x01FF;   // update x of marker
}


void ScreenBoardSelectionMenu::locateSelectionMarker() {
	uint16_t x = (LOCATIONS[this->selectionIndex].x << 3) - 10;
	uint16_t y = LOCATIONS[this->selectionIndex].y << 3;
	this->tiltManager.init(((int32_t) (x - 4)) << 16, ((int32_t) x) << 16, ((int32_t) 1) << 14, this);
	OAM[0] = (y & 0x00FF) | (((uint16_t) 1) << 13);
	OAM[1] = x & 0x01FF;
}


int16_t ScreenBoardSelectionMenu::drawText(int16_t x, int16_t y, const char *s) {
	if (x < 0)
		x = 15 - ((int16_t) (strlen(s) >> 1));   // 30 tiles width, so 15 tiles half-width
	uint16_t *dest = (MODE0_VRAM + 4096 + 1024) + (y << 5) + x;   // (y * 32) + x
	while (*s != 0) {
		*dest = (uint16_t) (*s - ' ');
		s++;
		dest++;
	}
	return x;
}


void ScreenBoardSelectionMenu::onLoad(InterScreenData *dataFromPreviousScreen) {
	this->interScreenData = dataFromPreviousScreen;
	GameStatus *gs = static_cast<GameStatus *>(this->interScreenData);
	// force blank
	DISPCNT = ((uint16_t) 1) << 7;
	// locate fonts at start of MODE0_VRAM (96 tiles)
	memcpy16(MODE0_VRAM, (const uint16_t *) FONT_1, FONT_SIZE >> 1);
	// generate aditional flat tiles (indices 96, 97, 98 and 99)
	uint16_t *dest = MODE0_VRAM + (FONT_SIZE >> 1);
	const uint16_t NUM_COLORS = 4;
	const uint16_t COLORS[NUM_COLORS] = { 0x5353, 0x2323, 0x8383, 0x9393 };
	const uint16_t *src = COLORS;
	for (uint16_t i = 0; i < NUM_COLORS; i++, dest += 32, src++)
		memset16(dest, *src, 32);
	// add tiles for "done" and "lock" marks (will be drawn at BG2)
	memcpy16(dest, LOCK_OBJ_TILE, 32);   // tile number 100
	dest += 32;
	memcpy16(dest, DONE_OBJ_TILE, 32);   // tile number 101
	const uint16_t LOCK_TILE_NUMBER = 100;
	const uint16_t DONE_TILE_NUMBER = 101;
	// there are 100 tiles, 100 * 64 = 6400 bytes, so map data should start at MODE0_VRAM + 8192 bytes (units of 2 Kb)
	dest = MODE0_VRAM + 4096;
	// fill pattern (32x32 tiles, 256x256 pixels)
	for (uint16_t y = 0; y < 32; y++) {
		uint16_t i = y & 0x0003;
		for (uint16_t x = 0; x < 32; x++, dest++, i++)
			*dest = 96 + (i & 0x0003);
	}
	// load BG palette
	memcpy16(BG_PALETTE, PALETTE, 256);
	// configure BG0: lowest priority (3), character base block = 0, 256 colors, screen base block = 4 (for 8192 bytes), size = 32x32 tiles = 32x32 words
	BG0CNT = ((uint16_t) 3) | (((uint16_t) 1) << 7) | (((uint16_t) 4) << 8);
	// configure BG0 movement with a circle path
	this->pathManager.init(CIRCLE_32_PATH, CIRCLE_32_PATH_SIZE, 2, *this);
	// draw text on BG1
	dest = MODE0_VRAM + 4096 + 1024;
	memset16(dest, 0, 1024);    // set tile 0 on all BG1 (it is a full transparent tile)
	char titleLabel[] = "File ?. Choose level";
	if (gs->saveFile.position == SaveFile::Position::FILE_1)
		titleLabel[5] = '1';
	else if (gs->saveFile.position == SaveFile::Position::FILE_2)
		titleLabel[5] = '2';
	else if (gs->saveFile.position == SaveFile::Position::FILE_3)
		titleLabel[5] = '3';
	//this->drawText(-1, 1, "Choose level");
	this->drawText(-1, 1, titleLabel);
	const Location *pLoc = LOCATIONS;       // draw level short names
	const Level *pLev = LevelManager::LEVELS;
	for (uint16_t i = 0; i < LevelManager::NUM_LEVELS; i++, pLoc++, pLev++)
		this->drawText(pLoc->x, pLoc->y, pLev->shortName);
	this->drawText(-1, 19, "Delete file");
	// configure BG1: low priority (2), character base block = 0, 256 colors, screen base block = 5 (2Kb after BG0), size = 32x32 tiles = 32x32 words
	BG1CNT = ((uint16_t) 2) | (((uint16_t) 1) << 7) | (((uint16_t) 5) << 8);
	// draw "lock" and "done" marks on BG2
	dest = MODE0_VRAM + 4096 + 1024 + 1024;
	memset16(dest, 0, 1024);    // set tile 0 on all BG2 (full transparent)
	pLoc = LOCATIONS;
	SaveFile::LevelStatus *pLevStatus = gs->saveFile.levels;
	for (uint16_t i = 0; i < LevelManager::NUM_LEVELS; i++, pLoc++, pLevStatus++) {
		if (*pLevStatus == SaveFile::LevelStatus::LOCKED) {
			// put "lock" tile at pLoc
			*(dest + (pLoc->y << 5) + pLoc->x) = LOCK_TILE_NUMBER;
		}
		else if (*pLevStatus == SaveFile::LevelStatus::FINISHED) {
			// put "done" tile at pLoc
			*(dest + (pLoc->y << 5) + pLoc->x) = DONE_TILE_NUMBER;
		}
	}
	BG2HOFS = ((uint16_t) -12) & 0x01FF;
	BG2VOFS = ((uint16_t) -4) & 0x01FF;
	// configure BG2: high priority (1), character base block = 0, 256 colors, screen base block = 6 (2Kb after BG1), size = 32x32 tiles = 32x32 words
	BG2CNT = ((uint16_t) 1) | (((uint16_t) 1) << 7) | (((uint16_t) 6) << 8);
	// configure OBJ palette for simple arrow marker
	OBJ_PALETTE[0] = 0;
	OBJ_PALETTE[1] = RGB555(0, 0, 15);
	OBJ_PALETTE[2] = RGB555(0, 30, 0);
	// configure OBJ tile data for simple arrow marker
	memcpy16((uint16_t *) OBJ_TILES_VRAM, OPTION_MARKER_OBJ_TILE, 32);
	// locate OBJ (arrow marker)
	OAM[2] = 512;
	this->selectionIndex = 0;
	this->locateSelectionMarker();
	// set mode 0 with BG0, BG1, BG2 and OBJ layers
	DISPCNT = (((uint16_t) 1) << 8) | (((uint16_t) 1) << 9) | (((uint16_t) 1) << 10) | (((uint16_t) 1) << 12);
	// initial status
	this->waitCounter = AFTER_INIT_WAIT_COUNTER_MAX;
	this->status = Status::WAIT;
}


Screen *ScreenBoardSelectionMenu::onVBlank(Keypad &keypad) {
	uint16_t keys = 0;
	if (!keypad.keyQueue.empty()) {
		keys = keypad.keyQueue.head();
		keypad.keyQueue.pop();
	}
	// update BG0 scroll from path manager
	this->pathManager.onVBlank();
	// check status
	if (this->status == Status::WAIT) {
		this->waitCounter--;
		if (this->waitCounter == 0)
			this->status = Status::IDLE;
	}
	else if (this->status == Status::IDLE) {
		if (keys & KEY_B)
			return this->backScreen;
		else if (keys != 0) {
			if (keys & KEY_RIGHT) {
				if (this->selectionIndex < LevelManager::NUM_LEVELS) {
					this->selectionIndex++;
					this->locateSelectionMarker();
				}
			}
			else if (keys & KEY_LEFT) {
				if (this->selectionIndex > 0) {
					this->selectionIndex--;
					this->locateSelectionMarker();
				}
			}
			else if (keys & KEY_UP) {
				int16_t upIndex = LOCATIONS[this->selectionIndex].upIndex;
				if (upIndex >= 0) {
					this->selectionIndex = upIndex;
					this->locateSelectionMarker();
				}
			}
			else if (keys & KEY_DOWN) {
				int16_t downIndex = LOCATIONS[this->selectionIndex].downIndex;
				if (downIndex >= 0) {
					this->selectionIndex = downIndex;
					this->locateSelectionMarker();
				}
			}
			else if (keys & KEY_A) {
				if (this->selectionIndex == LevelManager::NUM_LEVELS)
					return this->deleteFileScreen;
				else if (this->selectionIndex < LevelManager::NUM_LEVELS) {
					GameStatus *gs = static_cast<GameStatus *>(this->interScreenData);
					if (gs->saveFile.levels[this->selectionIndex] != SaveFile::LevelStatus::LOCKED) {
						gs->currentLevelIndex = this->selectionIndex;
						return this->boardSelectedScreen;
					}
				}
			}
			/*
			 * the key delay is managed on keypad object
			 *
			this->waitCounter = AFTER_KEY_WAIT_COUNTER_MAX;
			this->status = Status::WAIT;
			*/
		}
	}
	this->tiltManager.onVBlank();
	return this;
}


InterScreenData *ScreenBoardSelectionMenu::onUnload() {
	return this->interScreenData;
}
