

#include "ScreenTitle.H"
// #include "Ana.H"
#include "palette.H"
#include "fonts.H"
#include "gba.H"
#include "images.H"


using namespace avelino;
using namespace std;


void ScreenTitle::print(const char *text) {
	char *p = (char *) text;
	// calculate string length
	uint16_t n = 0;
	while (*p != 0) {
		p++;
		n++;
	}
	this->objUsed = n;
	// draw sprites as text
	const int16_t Y = 100;
	int16_t x = 120 - ((n << 3) >> 1);
	uint16_t i = 0;
	p = (char *) text;
	while (*p != 0) {
		OAM[i++] = Y | (((uint16_t) 1) << 13);
		OAM[i++] = x & 0x01FF;
		OAM[i++] = 512 + ((*p - ' ') << 1);
		i++;
		x += 8;
		p++;
	}
}


void ScreenTitle::onLoad(InterScreenData *dataFromPreviousScreen) {
	this->interScreenData = dataFromPreviousScreen;
	this->alpha = 0;
	this->objUsed = 0;
	this->timer = 5 * 60;   // 5 seconds
	// force blank to access VRAM
	DISPCNT = ((uint16_t) 1) << 7;
	// draw on framebuffer
	//memcpy16(MODE3_VRAM_FB, ANA, 240 * 160);
	memcpy16(MODE3_VRAM_FB, TITLE_RGB_IMAGE, 240 * 160);
	// configure OBJ 256 color palette (all colors white)
	memcpy16(OBJ_PALETTE, PALETTE, 256);
	// load OBJ tiles from FONT_1
	memcpy16((uint16_t *) OBJ_TILES_VRAM, (uint16_t *) FONT_1, 3072);
	// configure OBJs to print a text (this call updates this->objUsed to free the sprites on unload
	this->print("- Press START -");
	// 1st target is OBJ, 2nd target is BG2, fx = alpha blending
	BLDCNT = (((uint16_t) 1) << 4) | (((uint16_t) 1) << 6) | (((uint16_t) 1) << 10);
	// transparency at 50% (8/16 for both OBJ and BG2 intensities)
	BLDALPHA = (((uint16_t) this->alpha) << 8) | ((uint16_t) (16 - this->alpha));
	// enable mode 3 with background layer 2, enable obj, enable obj 1-d mapping
	DISPCNT = ((uint16_t) 3) | (((uint16_t) 1) << 10) | (((uint16_t) 1) << 12) | (((uint16_t) 1) << 6);
}


Screen *ScreenTitle::onVBlank(Keypad &keypad) {
	// recalculate sprite alpha blending
	this->alpha += ALPHA_INC;
	uint16_t alphaIntegerPart = this->alpha >> 8;
	if (alphaIntegerPart > 16) {
		this->alpha = 0;
		alphaIntegerPart = 0;
	}
	BLDALPHA = (((uint16_t) alphaIntegerPart) << 8) | ((uint16_t) (16 - alphaIntegerPart));
	// next screen when pressing START, else keep screen
	if (keypad.mask & KEY_START)
		return this->nextScreen;
	else
		return this;
}


InterScreenData *ScreenTitle::onUnload() {
	// free the sprites (OBJs) used
	uint16_t n = this->objUsed;
	uint16_t i = 0;
	while (n > 0) {
		OAM[i] = SCREEN_HEIGHT | (((uint16_t) 1) << 13);
		OAM[i + 1] = SCREEN_WIDTH & 0x01FF;
		i += 4;
		n--;
	}
	// disable alpha blending
	BLDCNT = 0;
	return this->interScreenData;
}

