

#include <stdint.h>
#include "Screen.H"
#include "ScreenTitle.H"
#include "ScreenMenu.H"
#include "ScreenFileSelectionMenu.H"
#include "ScreenBoardSelectionMenu.H"
#include "GameStatus.H"
#include "ScreenDeleteFileMenu.H"
#include "ScreenConfirmStartBoardMenu.H"
#include "ScreenBoard.H"
#include "ScreenPauseMenu.H"


using namespace std;
using namespace avelino;


typedef void (*fptr)(void);


#define  ISR_PTR   *((fptr *) 0x03007FFC)
#define  IME       *((volatile uint32_t *) 0x04000208)
#define  IE        *((volatile uint16_t *) 0x04000200)
#define  IF        *((volatile uint16_t *) 0x04000202)
#define  IFBIOS    *((volatile uint16_t *) 0x03007FF8)
#define  DISPSTAT  *((volatile uint16_t *) 0x04000004)


void isr()  __attribute__((target("arm")));


void isr() {
	// mark interrupt as served
	IF = 1;
	IFBIOS |= 1;
}


void enableInterrupts() {
	IME = 0;
	ISR_PTR = isr;
	DISPSTAT = ((uint16_t) 1) << 3;
	IE = 1;   // v-blank
	IME = 1;
}


int main() {
	enableInterrupts();
	GameStatus status;
	// define screens
	ScreenTitle st;
	ScreenFileSelectionMenu sfsm;
	ScreenBoardSelectionMenu sbsm;
	ScreenDeleteFileMenu sdfm;
	ScreenConfirmStartBoardMenu scsbm;
	ScreenBoard sb;
	ScreenPauseMenu spm;
	// link screens
	st.nextScreen = &sfsm;
	sfsm.backScreen = &st;
	sfsm.nextScreen = &sbsm;
	sbsm.backScreen = &sfsm;
	sbsm.deleteFileScreen = &sdfm;
	sbsm.boardSelectedScreen = &scsbm;
	sdfm.backScreen = &sbsm;
	sdfm.justDeletedScreen = &sfsm;
	scsbm.backScreen = &sbsm;
	scsbm.yesScreen = &sb;
	sb.pauseScreen = &spm;
	sb.afterFinishScreen = &sbsm;
	spm.continueScreen = &sb;
	spm.finishScreen = &sbsm;
	// start screen manager
	ScreenManager m;
	m.init(st, status);
	while (true) {
		asm volatile ("swi 0x05");    // wait for v-blank
		m.onVBlank();
	}
}
